%%% compare.m
%
%   Experiment to compare the performance of two W-transforms
%   in compressing a signal  x  which must exists before 
%   "compare" is invoked.
%
%   First W-transform is specified by  kw  (default QS)
%   Second W-transform by              kww (default D4)
%
%   x  is transformed into  [ y1, y2 ]  by each W-transform
%   and only the largest (in absolute value) fraction  r  of
%   the components of  y2  are retained.  The compressed
%   data is used to restore the signal (with and without 
%   orthogonal compensation by the first W-transform but only
%   without orthogonal compensation by the second W-transform).
%
%   The errors in the three cases are displayed, one line for
%   each choice of  r.  The choices of  r  are specified by the
%   variable  RR,  which can be altered before "compare" is
%   invoked.

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/11/94

if ~exist('RR'), RR = 0.05:0.05:1; end   % default choices of  r
if ~exist('kww'), kww=kwdau; end         % default 2nd W-transf

normx = norm(x);
[y1 y2] = kwt(x);
[d1 d2] = kwt(x,kww);

disp(' ')
disp('           W1OC        W1          W2')
disp(' ')
disp('  # Coef             errors')
disp(' ')
for r=RR

  cy2 = largesta(r,y2);
  cx = ikwt(y1,cy2);
  cE=cx-x;                 % error for  W1, without OC

  oy1 = oc(y1,y2-cy2);     % orthogonal compensation
  ox = ikwt(oy1,cy2);
  oE=ox-x;                 % error for  W1, with OC

  cd2 = largesta(r,d2);
  dx = ikwt(d1,cd2,kww);
  dE=dx-x;                 % error for  W2, without OC

% displaying errors
fprintf('   %3d   %f   %f   %f\n',nnz(cy2),norm(oE)/normx,norm(cE)/normx,norm(dE)/normx)

end
