function [X, cr, nz] = decp(Y1,Y2,Y3,Y4,r,cutoff)
% X = decp(Y1,Y2,Y3,Y4,q)    decompress the image contained
%     q > 1                  in the wavelet-decomposed Y1, Y2
%                            Y3, Y4, using q quantization levels
%
% X = decp(Y1,Y2,Y3,Y4,r)    by keeping r (e.g. 0.1) of the
%     r < 1                  largest components in Y2, Y3, Y4
%
% X = decp(Y1)               by using Y2=Y3=Y4=0.
%
% [X, cr, nz] = decp(...)    nz = total number of nonzero components
%                            in Y1, Y2, Y3, Y3.
%
%                            cr = size of X / nz

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/11/94

s = size(Y1);
if nargin == 1, Y2 = zeros(s); Y3 = Y2; Y4 = Y2; 
elseif nargin >=5 & r <= 1 , 
                    Y = [ 0*Y1 Y2; Y3 Y4 ];
                    Y = largesta(r,Y);
  nz = prod(size(Y1))+nnz(Y);
                    Y(1:s(1),1:s(2)) = Y1;
elseif nargin >= 5, Y2 = quant(Y2,r); 
                    Y3 = quant(Y3,r); 
                    Y4 = quant(Y4,2*r); 
                 if nargin > 5
                    Y2 = Y2.*(abs(Y2)>cutoff*r);
                    Y3 = Y3.*(abs(Y3)>cutoff*r);
                    Y4 = Y4.*(abs(Y4)>2*cutoff*r);
                 end
  nz = prod(size(Y1))+nnz(Y2)+nnz(Y3)+nnz(Y4);
  Y = [Y1 Y2;Y3 Y4];
end

X = ikwt2(Y);
cr = 4*s(1)*s(2)/nz;
