%%% ikwt.m

% inverse W-transform
%
% x = ikwt(y)        if y is a vector, x is the 1D inv. W-transform of x
%                       using the default W-transf.
%                    if y is a matrix, x is the 2D inv. W-transform of x
% x = ikwt(y,k)      using the W-transform specified by  k
%
% x = ikwt(y1,y2)    if y1, y2 are vectors and y2 is longer than 6, x is
%                       the 1D inv. W-transf. of [y1 y2]
%                    if the second argument = 0, y2 is assumed
%                       to be 0*y1.
%
% x = ikwt(y1,y2,y3,y4)     y1, y2, y3, y4 are matrices, and x is the
%                           inv. W-transf. of [y1 y2; y3 y4]
%
% x = ikwt(y1,y2,y3,y4,k)   using the W-transform specified by  k

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/11/94

function y=ikwt(x1,x2,x3,x4,k)

ss=size(x1); 
if nargin == 1
  if ss(1) == 1, x = x1(:); 
  elseif ss(2) > 1 & nargin == 1, y = ikwt2(x1); return
  else x = x1;
  end
elseif nargin == 2
  if min(ss) > 1
    if length(x2(:)) == 1 & x2(1) == 0
      y = ikwt2([x1 0*x1; 0*x1 0*x1]);
    else
      y = ikwt2(x1,x2); 
    end
    return
  elseif size(x2,1) == 1 & size(x2,2) < 7 & size(x2,2) > 3
    x = x1(:); k = x2;
  else 
    if length(x2) == 1 & x2(1) == 0, x2 = 0*x1; end
    x = [x1(:); x2(:)];
  end
elseif nargin == 3
  if length(x2) == 1 & x2(1) == 0, x2 = 0*x1; end
  if min(ss) > 1
    y = ikwt2([x1 x2; x2 x2],x3); return
  else
    x = [x1(:); x2(:)]; k = x3;
  end
elseif nargin == 4
  y = ikwt2([x1 x2; x3 x4]); return
elseif nargin == 5
  y = ikwt2([x1 x2; x3 x4],k); return
end

if exist('k') ~=1
  global kw
  if exist('kw') ~= 1, k = [1 3 3 1]/4; else k = kw; end
end

if length(k) == 4, c=1; d=-1;
elseif length(k) == 5, c=k(5); d=-1;
elseif length(k) == 6, c=k(5); d = k(6); 
else   k = [1 3 3 1]/4; c=1; d=-1;
end

C=-k(4)/c; D=k(3)/c; A=-k(2)/d; B=k(1)/d; 

DD = (A*D-B*C); 

s=size(x); n=floor(s(1)/2);
if s(1) == n*2
  x1=x(1:n,:);
  x2=x(n+1:s(1),:);

  y1=[ (d*x1(1,:)-x2(1,:))/(C+D)
       (C*(c*x1(1:n-1,:)-x2(1:n-1,:))+A*(d*x1(2:n,:)-x2(2:n,:)))/DD ]/(d-c);

  y2=[ (D*(-c*x1(1:n-1,:)+x2(1:n-1,:))+B*(-d*x1(2:n,:)+x2(2:n,:)))/DD
       (-c*x1(n,:)+x2(n,:))/(A+B) ]/(d-c);

  y=zeros(s);
  y(1:2:2*n,:)=y1; y(2:2:s(1),:)=y2;

else
  x1=x(1:n+1,:);
  x2=x(n+2:s(1),:);

  y1=[ (d*x1(1,:)-x2(1,:))/(C+D)
       (C*(c*x1(1:n-1,:)-x2(1:n-1,:))+A*(d*x1(2:n,:)-x2(2:n,:)))/DD ]/(d-c);
  DDD = A*D+A^2+A*B-B*C;
  y1=[y1; (C*(c*x1(n,:)-x2(n,:))/(d-c)+A*x1(n+1,:))/DDD];

  y2=(D*(-c*x1(1:n-1,:)+x2(1:n-1,:))+B*(-d*x1(2:n,:)+x2(2:n,:)))/DD/(d-c);
  y2=[y2; (-(D+A+B)*(c*x1(n,:)-x2(n,:))/(d-c)-B*x1(n+1,:))/DDD];

  y=zeros(s);
  y(1:2:s(1),:)=y1; y(2:2:s(1),:)=y2;

end

if ss(1) == 1, y = y'; end
