% x = ikwt2(y)          2D inverse W-transform
% x = ikwt2(y,k)

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/11/94

function z = ikwt2(x,k)

if exist('k') ~= 1
  global kw
  if exist('kw') ~= 1, k = [1 3 3 1]/4; else k = kw; end
end

if length(k) == 4, c=1; d=-1;
elseif length(k) == 5, c=k(5); d=-1;
elseif length(k) == 6, c=k(5); d = k(6); 
else   k = [1 3 3 1]/4; c=1; d=-1;
end

C=-k(4)/c; D=k(3)/c; A=-k(2)/d; B=k(1)/d; 

DD = (A*D-B*C); 

s=size(x); n=floor(s(1)/2);
if s(1) == n*2
  x1=x(1:n,:);
  x2=x(n+1:s(1),:);

  y1=[ (d*x1(1,:)-x2(1,:))/(C+D)
       (C*(c*x1(1:n-1,:)-x2(1:n-1,:))+A*(d*x1(2:n,:)-x2(2:n,:)))/DD ]/(d-c);

  y2=[ (D*(-c*x1(1:n-1,:)+x2(1:n-1,:))+B*(-d*x1(2:n,:)+x2(2:n,:)))/DD
       (-c*x1(n,:)+x2(n,:))/(A+B) ]/(d-c);

  y=zeros(s);
  y(1:2:2*n,:)=y1; y(2:2:s(1),:)=y2;

else
  x1=x(1:n+1,:);
  x2=x(n+2:s(1),:);

  y1=[ (d*x1(1,:)-x2(1,:))/(C+D)
       (C*(c*x1(1:n-1,:)-x2(1:n-1,:))+A*(d*x1(2:n,:)-x2(2:n,:)))/DD ]/(d-c);
  DDD = A*D+A^2+A*B-B*C;
  y1=[y1; (C*(c*x1(n,:)-x2(n,:))/(d-c)+A*x1(n+1,:))/DDD];

  y2=(D*(-c*x1(1:n-1,:)+x2(1:n-1,:))+B*(-d*x1(2:n,:)+x2(2:n,:)))/DD/(d-c);
  y2=[y2; (-(D+A+B)*(c*x1(n,:)-x2(n,:))/(d-c)-B*x1(n+1,:))/DDD];

  y=zeros(s);
  y(1:2:s(1),:)=y1; y(2:2:s(1),:)=y2;

end

n=floor(s(2)/2);
if s(2) == n*2
  y1=y(:,1:n);
  y2=y(:,n+1:s(2));

  z1=[ (d*y1(:,1)-y2(:,1))/(C+D) ...
       (C*(c*y1(:,1:n-1)-y2(:,1:n-1))+A*(d*y1(:,2:n)-y2(:,2:n)))/DD ]/(d-c);

  z2=[ (D*(-c*y1(:,1:n-1)+y2(:,1:n-1))+B*(-d*y1(:,2:n)+y2(:,2:n)))/DD ...
       (-c*y1(:,n)+y2(:,n))/(A+B) ]/(d-c);

  z=zeros(s);
  z(:,1:2:2*n)=z1; z(:,2:2:s(2))=z2;

else
  y1=y(:,1:n+1);
  y2=y(:,n+2:s(2));

  z1=[ (d*y1(:,1)-y2(:,1))/(C+D) ...
       (C*(c*y1(:,1:n-1)-y2(:,1:n-1))+A*(d*y1(:,2:n)-y2(:,2:n)))/DD ]/(d-c);
  DDD = A*D+A^2+A*B-B*C;
  z1=[z1 (C*(c*y1(:,n)-y2(:,n))/(d-c)+A*y1(:,n+1))/DDD];

  z2=(D*(-c*y1(:,1:n-1)+y2(:,1:n-1))+B*(-d*y1(:,2:n)+y2(:,2:n)))/DD/(d-c);
  z2=[z2  (-(D+A+B)*(c*y1(:,n)-y2(:,n))/(d-c)-B*y1(:,n+1))/DDD];

  z=zeros(s);
  z(:,1:2:s(2))=z1; z(:,2:2:s(2))=z2;

end

z=zeros(s);
z(:,1:2:s(2))=z1; z(:,2:2:s(2))=z2;
