%%% imcomp.m
%
%   Experiment to test the 3-level compression algorithm 
%   for a b/w image stored in  L.
%
%   PREPARATION: (for a ready example, issue the command)
%
%            load ldemo
%
%         o  L  must already have the image.
%
%         o  Do a "wmai" to get a 3 or 4 level multiresolution
%            analysis (enter L and then 3 or 4 at the prompts)
%
%   USAGE:   imcomp
%
%         o  Current or default  r3  will be displayed.
%
%         o  Enter new  r3  (<CR> to use the same  r3)
%
%   Default  r3  is  "32  16  8"  which means that 
%
%   1st level wavelet matrices are quantized by 32, 32, 2*32,
%   2nd level                                   16, 16, 2*16,
%   3rd level                                    8,  8, 2*8

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/18/94

if exist('IMCOMPR') ~= 1
if ~exist('r3'), r3 = [ 32 16 8 ]; end
if ~exist('cutoff'), cutoff=[ 0.1 0.1 0.1 ]; end
fprintf('current r3 = ')
fprintf(' %f ',r3)
tr3 = input('Enter new r3: ','s');
if length(tr3) > 0
  eval([ 'tr3 = [' tr3 '];']);
  if length(tr3) == 2
    eval([ 'r3(round(' num2str(tr3(1)) ')) = ' num2str(tr3(2)) ';'])
  else r3 = tr3;
  end
end
else
  r3 = IMCOMPR;
end

kwold = kw;
kw = kwqs;

LEVELS = length(r3);

if LEVELS == 4
[cWL31 cr4 nz4] = qcp(WL41,WL42,WL43,WL44,r3(4));
nz4 = nz4 - prod(size(WL41)); 
disp([ 'Level 4 decompression done   nz = ' int2str(nz4) ])
else
cWL31 = WL31;
end
[cWL21 cr3 nz3] = qcp(cWL31,WL32,WL33,WL34,r3(3),cutoff(3));
nz3 = nz3 - prod(size(WL31)); 
disp([ 'Level 3 decompression done   nz = ' int2str(nz3) ])
[cWL11 cr2 nz2] = qcp(cWL21,WL22,WL23,WL24,r3(2),cutoff(2));
nz2 = nz2 - prod(size(cWL21)); 
disp([ 'Level 2 decompression done   nz = ' int2str(nz2) ])
[cL cr1 nz1] = qcp(cWL11,WL12,WL13,WL14,r3(1),cutoff(1));
nz1 = nz1 - prod(size(cWL11)); 
disp([ 'Level 1 decompression done   nz = ' int2str(nz1) ])

  magf = log(sum(kw))/log(2);
  cr = 8*prod(size(cL))*log(2)/(prod(size(WL31))*(1+log(max(WL31(:)))) + nz3*(1+log(max([WL32(:);WL33(:)]))-log(r3(3))) + nz2*(1+log(max([WL22(:);WL23(:)]))-log(r3(2))) + nz1*(1+log(max([WL12(:);WL13(:)]))-log(r3(1))));
  fprintf('\ncr = %f\n',cr)
TITLE = [ 'Quantized Image, cr = ' num2str(cr) '  (' int2str(r3(1)) ', ' int2str(r3(2)) ', ' int2str(r3(3)) ')' ];
XLABEL = [ int2str(nz1) '   ' int2str(nz2) '   ' int2str(nz3) ];

image(max(10,cL)), colormap(gray(256))
axis image
title(TITLE)
xlabel(XLABEL)
% if exist('IMCPMPR') ~= 1
  % saveraw(cL)
% end

kw = kwold;
