function [y, yw, y3, y4]= kwt(x,k)
% W-matrix transform
%
% y = kwt(x)       if x is a vector, y is the 1D transform
%                  if x is a matrix, y is the 2D transform
% [y1 y2] = kwt(x) if x is a vector, y1 and y2 are the odd and even
%                     components, respectively, of y
%
% [y1 y2 y3 y4] = kwt(x)    4 parts of the 2D transform of matrix x
%
% The default transform is controlled by the global variable  kw, 
% initially set to be the Quadratic Spline Transform in  startup.m.  
%
%   kw = kwdau     sets the default to Daubechies


if nargin == 1
  global kw
  if exist('kw') ~= 1, k = [1 3 3 1]/4; else k = kw; end
end

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/18/94

ss=size(x); 
if ss(1) == 1, x = x(:); 
elseif ss(2) > 1 
  if nargout == 4, [y, yw, y3, y4] = kwt2(x,k);
  else y = kwt2(x,k); end
  return
end

if length(k) == 4, c=1; d=-1;
elseif length(k) == 5, c=k(5); d=-1;
elseif length(k) == 6, c=k(5); d = k(6); 
else   k = [1 3 3 1]/4; c=1; d=-1;
end

C=-k(4)/c; D=k(3)/c; A=-k(2)/d; B=k(1)/d; 

s=size(x);
n=floor(s(1)/2);
x1=x(1:2:s(1),:);
x2=x(2:2:s(1),:);

if s(1) == 2*n
y=[ (C+D)*x1(1,:)+A*x2(1,:)+B*x1(2,:)
     C*x2(1:n-2,:)+D*x1(2:n-1,:)+A*x2(2:n-1,:)+B*x1(3:n,:)
     C*x2(n-1,:)+D*x1(n,:)+(A+B)*x2(n,:) ];

yw=[ c*(C+D)*x1(1,:)+d*(A*x2(1,:)+B*x1(2,:))
     c*(C*x2(1:n-2,:)+D*x1(2:n-1,:))+d*(A*x2(2:n-1,:)+B*x1(3:n,:))
     c*(C*x2(n-1,:)+D*x1(n,:))+d*(A+B)*x2(n,:) ];
else
y=[ (C+D)*x1(1,:)+A*x2(1,:)+B*x1(2,:)
     C*x2(1:n-1,:)+D*x1(2:n,:)+A*x2(2:n,:)+B*x1(3:n+1,:)
     C*x2(n,:)+(A+B+D)*x1(n+1,:) ];

yw=[ c*(C+D)*x1(1,:)+d*(A*x2(1,:)+B*x1(2,:))
     c*(C*x2(1:n-1,:)+D*x1(2:n,:))+d*(A*x2(2:n,:)+B*x1(3:n+1,:)) ];
end

if nargout <= 1
  if ss(1) == 1, y = [y' yw'];
  else y = [y; yw]; end
elseif ss(1) == 1,
  y = y'; yw = yw';
end
