function [z1, z2, z3, z4] = kwt2(x,k)
% 2D kwong-wavelet transform

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/18/94

if nargin == 1
  global kw
  if exist('kw') ~= 1, k = [1 3 3 1]/4; else k = kw; end
end

if length(k) == 4, c=1; d=-1;
elseif length(k) == 5, c=k(5); d=-1;
elseif length(k) == 6, c=k(5); d = k(6); 
else   k = [1 3 3 1]/4; c=1; d=-1;
end

C=-k(4)/c; D=k(3)/c; A=-k(2)/d; B=k(1)/d; 

s=size(x); m=floor(s(1)/2); n=floor(s(2)/2);

x1=x(1:2:s(1),:);
x2=x(2:2:s(1),:);

if s(1) == 2*m
y=[ (C+D)*x1(1,:)+A*x2(1,:)+B*x1(2,:)
     C*x2(1:m-2,:)+D*x1(2:m-1,:)+A*x2(2:m-1,:)+B*x1(3:m,:)
     C*x2(m-1,:)+D*x1(m,:)+(A+B)*x2(m,:) ];

yw=[ c*(C+D)*x1(1,:)+d*(A*x2(1,:)+B*x1(2,:))
     c*(C*x2(1:m-2,:)+D*x1(2:m-1,:))+d*(A*x2(2:m-1,:)+B*x1(3:m,:))
     c*(C*x2(m-1,:)+D*x1(m,:))+d*(A+B)*x2(m,:) ];
else
y=[ (C+D)*x1(1,:)+A*x2(1,:)+B*x1(2,:)
     C*x2(1:m-1,:)+D*x1(2:m,:)+A*x2(2:m,:)+B*x1(3:m+1,:)
     C*x2(m,:)+(A+B+D)*x1(m+1,:) ];

yw=[ c*(C+D)*x1(1,:)+d*(A*x2(1,:)+B*x1(2,:))
     c*(C*x2(1:m-1,:)+D*x1(2:m,:))+d*(A*x2(2:m,:)+B*x1(3:m+1,:)) ];
end

y = [ y; yw]; 

y1=y(:,1:2:s(2));
y2=y(:,2:2:s(2));

if s(2) == 2*n
z=[ (C+D)*y1(:,1)+A*y2(:,1)+B*y1(:,2) ...
     C*y2(:,1:n-2)+D*y1(:,2:n-1)+A*y2(:,2:n-1)+B*y1(:,3:n) ...
     C*y2(:,n-1)+D*y1(:,n)+(A+B)*y2(:,n) ];

zw=[ c*(C+D)*y1(:,1)+d*(A*y2(:,1)+B*y1(:,2)) ...
     c*(C*y2(:,1:n-2)+D*y1(:,2:n-1))+d*(A*y2(:,2:n-1)+B*y1(:,3:n)) ...
     c*(C*y2(:,n-1)+D*y1(:,n))+d*(A+B)*y2(:,n) ];
else
z=[ (C+D)*y1(:,1)+A*y2(:,1)+B*y1(:,2) ...
     C*y2(:,1:n-1)+D*y1(:,2:n)+A*y2(:,2:n)+B*y1(:,3:n+1) ...
     C*y2(:,n)+(A+B+D)*y1(:,n+1) ];

zw=[ c*(C+D)*y1(:,1)+d*(A*y2(:,1)+B*y1(:,2)) ...
     c*(C*y2(:,1:n-1)+D*y1(:,2:n))+d*(A*y2(:,2:n)+B*y1(:,3:n+1)) ];
end

z1 = [ z zw]; 

if nargout == 4,
   m = ceil(s(1)/2); n = ceil(s(2)/2);
   z2 = z1(1:m,n+1:s(2));
   z3 = z1(m+1:s(1),1:n);
   z4 = z1(m+1:s(1),n+1:s(2));
   z1 = z1(1:m,1:n);
end
