% [ y n ] = largesta(n,x)  return a matrix y containing 
%           n > 1          the largest (absolute value) n
%                          elements of x.
%
%           largesta(r,x)  (approximately the fraction) r of
%           r <= 1         the largest (absolute value) 
%                          elements of x
%
% Example:  largesta(0.1,x)  gives the largest 10 elements
%           largesta(0.1,x)  gives the largest 1/10 of the elements 
%
% SEE: largest, smallest

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 10/26/93

function [ y, N ] = largest(n,x)

if length(n) > 1, tmp = n; n = x(1); x = tmp; end
ax = abs(x); 
M = max(max(ax)); 
a = 0; b = M; n1 = prod(size(x(:))); n2 = nnz(ax == M);
if  n == 1 | n == n1, y = x; n = n1; return
elseif n <= 0, y = 0*x; n = 0; return
elseif n < 1, n = min(n1-1,n1*n); 
end

while n1 ~= n & n2 ~= n & n1 ~= n2 & b-a > 0.000001*M
  c = (a+b)/2;
  nc = nnz(ax >= c);
  if nc > n, a = c; n1 = nc;
  else       b = c; n2 = nc;
  end
end

if n1 == n
  small = find(ax < b);
  % y = x.*(ax > b);
else
  small = find(ax < c);
  % y = x.*(ax > c);
end
y = x;
y(small) = zeros(size(small));
N = nnz(y);
