%%% ldemo        Compressed Lena Demo

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 9/20/94

echo off
clc
disp(' ')
disp('   ****************  COMPRESSED  LENA  DEMO  ******************')
disp(' ')
disp('** This demo shows a 10-fold compressed image of Lena.')
disp(' ')
disp('** The compressed data is represented by the matrices:')
disp(' ')
disp('     WL31  WL32  WL33  WL34;  WL22  WL23  WL24;  WL12  WL13  WL14')
disp(' ')
disp('   which is the 3-level "quadratic spline" W-multiresolution analysis')
disp('   of the approximate restored image.')

disp(' ')
load ldemo
disp('                                           [press any key to continue]')
disp(' ')
pause
disp('   ****************  COMPRESSED  LENA  DEMO  ******************')
disp(' ')
disp('** Compression is achieved by quantizing the matrices at various levels.')
disp('   The original image is stored in  L.  The above matrices are ')
disp('   constructed from  L  by using the command  "wmai"  and then')
disp('   quantizing the resulting matrices with the commands')
disp('   "WL32 = quant(WL32,8)", etc.')
disp(' ')
disp('** The quantization levels used are:')
disp(' ')
disp('       8    8    16    16    16    32    32    32    32    64')
disp(' ')
disp('   respectively.')
disp(' ')
disp('                                           [press any key to continue]')
disp(' ')
pause
disp('   ****************  COMPRESSED  LENA  DEMO  ******************')
disp(' ')
disp('** A rough estimate of the compression ratio can be obtained by finding')
disp('  the total number of nonzero elements in the matrices.')

fprintf('\n     Number of nonzero elements in the WL''s   =   25664\n')
fprintf('\n     Total # of elements in the original image =  262144\n')
fprintf('\n     Compression Ratio CR is approximately     =  262144 / 25664  = 10.21\n')

disp(' ')
disp('** The distribution of nonzero elements in the multiresolution analysis')
disp('   is shown in the graph.')
disp(' ')
figure('pos',[0 400 420 420])
spy([ [ [ WL31 WL32; WL33 WL34] WL22; WL23 WL24 ] WL12; WL13 WL14 ])
title('Distribution of nonzero elements in the WL''s')
disp('                                           [press any key to continue]')
disp(' ')
pause
disp('   ****************  COMPRESSED  LENA  DEMO  ******************')
disp(' ')
disp('The precise compression ratio can only be found after the following')
disp('considerations (we believe that the CR  will be higher):')
disp(' ')
disp('  o  The locations of the nonzero elements must be recorded -- usually')
disp('     done using run length coding.  This step will increase the data ')
disp('     and hence decrease the ratio.')
disp(' ')
disp('  o  Quantization means requiring less bits to store each matrix')
disp('     element, hence, increasing the ratio.')
disp(' ')
disp('  o  Entropy coding can further enhance the ratio.')
disp(' ')
disp('                                           [press any key to continue]')
disp(' ')
pause
disp('   ****************  COMPRESSED  LENA  DEMO  ******************')
disp(' ')
disp('The following commands restore and display the compressed image:')

echo on

   cWL21 = ikwt(WL31, WL32, WL33, WL34);   % 1st-level inverse W-transf.
   cWL11 = ikwt(cWL21, WL22, WL23, WL24);  % 2nd-level, etc.
   cL    = ikwt(cWL11, WL12, WL13, WL14);  % or more concisely, if you want
%  WL = [[[WL31 WL32; WL33 WL34] WL22; WL23 WL24] WL12; WL13 WL14];
%  cL = iwma(WL);                          % inv. multi. resol. anal.

   colormap(gray(256)), imagesc(cL), axis('image')

echo off

disp(' ')
disp('   ************************  THE  END  ************************')
disp(' ')
disp('For more experiments on image compression, see "imcomp"')
disp(' ')

title('Lena from 10-fold QS-transform compressed data')

