function y = oc(y1,ye,k)
% y = oc(y1,ye)   orthogonal compensation for the k-wavelet transform
%                 ye  is the error incurred by throwing away part of the
%                 y2  vector in the wavelet decomposition.  The orthogonal
%                 projection of  ye  is computed and added to  y1 to
%                 obtain the best approximation  y.

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/18/94

if nargin == 2
  global kw
  if exist('kw') ~= 1, k = [1 3 3 1]/4; else k = kw; end
end

if length(k) == 4, c=1; d=-1;
elseif length(k) == 5, c=k(5); d=-1;
elseif length(k) == 6, c=k(5); d = k(6); 
else   k = [1 3 3 1]/4; c=1; d=-1;
end

% C=-k(4)/c; D=k(3)/c; A=-k(2)/d; B=k(1)/d; 
k1=k(1); k2=k(2); k3=k(3); k4=k(4);
h1=-k1/d; h2=-k2/d; h3=-k3/c; h4=-k4/c;

s1 = size(y1); se = size(ye);
y1 = y1(:); ye = ye(:);
m = length(y1); n = length(ye);

A = k1*k3+k2*k4;
B = k1^2+k2^2+k3^2+k4^2; 
C = k3*k1+k4*k2; 
B1 = (k2*k3-k1*k4)^2/(k2-k1)^2 +k3^2+k4^2; 
Bn = k1^2+k2^2+(k2*k3-k1*k4)^2/(k4-k3)^2;

D = k1*h3+k2*h4;
E = k1*h1+k2*h2+k3*h3+k4*h4; 
F = k3*h1+k4*h2; 
E1 = -((k2*k3-k1*k4)/(k2-k1)/c)^2*d+k3*h3+k4*h4; 

if m == n
  En = k1*h1+k2*h2-((k2*k3-k1*k4)/(k4-k3)/d)^2*c; 
  z = ([ 0;D*ye(1:n-1) ] + [ E1*ye(1);E*ye(2:n-1);En*ye(n) ] + [ F*ye(2:n);0 ]);
  GH = toeplitz([ B A zeros(1,n-2) ],[ B C zeros(1,n-2) ]);
  GH(1,1) = B1;  GH(n,n) = Bn;
else

  r = -h2*h3+h1*h4;
  r = (r/(r-h1*h2+h2^2))^2;
  Dn = (c-d)*r*( (h1-h2+h3)*h1 - h2*k4 );
  En = k1*h1+k2*h2-r*c*( (h1-h2+h3)^2 + (k4)^2 );

  z = ([ 0;D*ye(1:n-1);Dn*ye(n) ] + [ E1*ye(1);E*ye(2:n-1);En*ye(n);0 ] + [ F*ye(2:n);0;0 ]);

  GH = toeplitz([ B A zeros(1,n-1) ],[ B C zeros(1,n-1) ]);
  GH(1,1) = B1;  

  GH(n,n) = k1^2+k2^2+r*c^2*( (h1-h2+h3)^2 + (k4)^2 );
  GH(n+1,n+1) = r*(d-c)^2*(h1^2+h2^2);
  GH(n,n+1) = -c*Dn;
  GH(n+1,n) = GH(n,n+1);
end

y = y1+GH\z;
if s1(1) ==1, y = y'; end
