function [X, cr, nz] = qcp(Y1,Y2,Y3,Y4,r,cutoff)
% X = qcp(Y1,Y2,Y3,Y4,q)     decompress the image contained
%                            in the wavelet-decomposed Y1, Y2
%                            Y3, Y4, using q quantization levels
%
% [X, cr, nz] = decp(...)    nz = total number of nonzero components
%                            in Y1, Y2, Y3, Y3.
%
%                            cr = size of X / nz

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/11/94

s = size(Y1);
if nargin == 1, Y2 = zeros(s); Y3 = Y2; Y4 = Y2; end
          Y2 = quant(Y2,r); 
          Y3 = quant(Y3,r); 
          Y4 = quant(Y4,2*r); 
       if nargin > 5
          Y2 = Y2.*(abs(Y2)>cutoff*r);
          Y3 = Y3.*(abs(Y3)>cutoff*r);
          Y4 = Y4.*(abs(Y4)>2*cutoff*r);
       end
  nz = prod(size(Y1))+nnz(Y2)+nnz(Y3)+nnz(Y4);
  Y = [Y1 Y2;Y3 Y4];

X = ikwt2(Y);
cr = 4*s(1)*s(2)/nz;
