%%% wdemo        W-Transform Demo 

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/18/94

format compact
echo on

%   ****************  W-transform  DEMO  ******************

%% Let us build a few sample signals:

   t = 0:.01:1;
   T = t(1:5:101);
   x = t .* sin(20*t);            % 1D sample signal
   [X Y] = meshdom(T,T);
   L = sin(X).*cos(2*Y);          % 2D sample signal

%% The default QS transform is obtained by:

   y = kwt(x);                    % kwt  automatically knows
   M = kwt(L);                    % whether to use 1D or 2D transf.

%% Plot the original and the transformed signals  x  and  y:

   figure(gcf)
   subplot(2,1,1), plot(x), title('Original signal x')
   subplot(2,1,2), plot(y,'r'), title('Transformed signal y')

                                pause % [press any key to continue])

%   ****************  W-transform  DEMO  ******************

%% The low- and high-pass components of the transformed signal
%%   can be extracted directly using the command.

   [y1 y2] = kwt(x);
   figure(gcf), plot(1:51,y1,'r',52:101,y2,'c')
   title('Transformed signal y1 and y2')

%% See the new graph.
                                pause % [press any key to continue])

%   ****************  W-transform  DEMO  ******************

%% Inverse QS transform is

   rx = ikwt(y);                  % restored-x
   norm(x-rx)                     % error between x and rx

                                pause % [press any key to continue])

%   ****************  W-transform  DEMO  ******************

%% Compression is achieved by throwing away small components of y2

   cy2 = largesta(7,y2);          % retain the 7 largest (abs. val.)
                                  % components of  y2
   cx  = ikwt(y1,cy2);            % compressed-x
   subplot(2,1,1), hold on, plot(cx-.2,'g'), hold off
   norm(x-cx)/norm(x)             % relative error between x and cx

%% or by quantizing

   cx = ikwt(y1,quant(y2,0.005)); % rounding to the nearest 0.005
   norm(x-cx)/norm(x)             % relative error between x and cx

%% The restored signal is plotted along side the original  x  in the
%%   first graph.

                                pause % [press any key to continue])

%   ****************  W-transform  DEMO  ******************

%% Multiresolution analysis is performed by the commands

   z3 = wma(x);                   % default 3-level
   z4 = wma(x,4);                 % 4-level

%% Individual wavelet components can be extracted by

%  w2 = maw(z4,2);                % 2nd-level wavelet
%  y1 = maw(z4,4,1);              % 4th-level y1

%% The multiresolution analysis can be visualized by plotting the
%%   low-pass component at the top level and the wavelet components
%%   at the other levels using the command:

%  wplot(z3)                      % default # of level is 3
   wplot(z4,4,0.03)               % 4-level; 0.03 is a magnification
                                  % factor (default = 1) - the 
                                  % wavelet coef. are usually too small
                                  % to show up.

                                pause % [press any key to continue])

%   ****************  W-transform  DEMO  ******************

%% The default W-transform is controlled by the global variable  kw,
%%   which is initially set to give the QS (quadratic spline) tranf.
%%   A different W-transform can be request either by changing  kw
%%   or by an additional argument to  kwt, e.g.

   [dy1 dy2] = kwt(x,kwdau);      % uses the Daubechies-4 transf.

%% The restored signal on compressing  x  by  kwdau, keeping 7 
%%   components of  dy2  is

   dx = ikwt(dy1,largesta(7,dy2),kwdau);
   norm(x-dx)/norm(x)             % relative error between x and dx


%% FURTHER TOPICS:                   SEE:

%%   orthogonal compensation         oc
%%   image compression               ldemo  imcomp  decp
%%   comparing W-transforms          compare  opt
%%   scaling and wavelet functions   sf  wf  sdil

%   ****************  W-transform  DEMO  ******************


echo off
format loose
