function [X0,bX1,bX2] = wf(k)
% [X, bX1 bX2] = wf(k)    
%
%   Computes and plots the wavelet function of the W-transform
%     corresponding to the parameter vector k (default is in  
%     kw  - initially set to give the QS transform.)
%
%   CAUTION: No output will be produced if  wf  is not assigned 
%   -------    to a variable
%
%   Only  X,  the regular interior wavelet function is plotted.
%   bX1 and bX2 are the left and right boundary wavelet functions.

% kwong@mcs.anl.gov (Man Kam Kwong, MCS, Argonne National Laboratory)
% dritz@mcs.anl.gov (Ken Dritz)
% Last revised: 8/18/94

if nargin == 0 
  global kw
  if exist('kw') ~= 1, k = [1 3 3 1]/4; else k = kw; end
end

X=ikwt([0 0 0 0 1 0]',k);
X1=ikwt([0 0 1 0]',k);
X2=ikwt([0 0 0 1]',k);

for i=1:9
  fprintf('%3d',i)
  X = ikwt([ X; zeros(size(X)) ],k);
  if nargout > 1
    X1 = ikwt([ X1; zeros(size(X1)) ],k);
    X2 = ikwt([ X2; zeros(size(X1)) ],k);
  end
end
fprintf('\n')
if nargout > 0
  X0 = X/max(X); X1 = X1/max(X1); X2 = X2/max(X2);
end
plot(X)
